## begin license ##
#
#    "Meresco Harvester" consists of two subsystems, namely an OAI-harvester and
#    a web-control panel.
#    "Meresco Harvester" is originally called "Sahara" and was developed for
#    SURFnet by:
#        Seek You Too B.V. (CQ2) http://www.cq2.nl
#    Copyright (C) 2006-2007 SURFnet B.V. http://www.surfnet.nl
#    Copyright (C) 2007-2008 SURF Foundation. http://www.surf.nl
#    Copyright (C) 2007-2010 Seek You Too (CQ2) http://www.cq2.nl
#    Copyright (C) 2007-2009 Stichting Kennisnet Ict op school.
#       http://www.kennisnetictopschool.nl
#    Copyright (C) 2009 Tilburg University http://www.uvt.nl
#    Copyright (C) 2010 Stichting Kennisnet http://www.kennisnet.nl
#
#    This file is part of "Meresco Harvester"
#
#    "Meresco Harvester" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Meresco Harvester" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Meresco Harvester"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##

import time, os, sys, re, string
if sys.version_info[:2] == (2,3):
    from sets import Set as set
from eventlogger import EventLogger
from ids import Ids
import traceback
from os.path import join as pathjoin, isdir
from os import makedirs
from state import State

def idfilename(stateDir, repositorykey):
    return pathjoin(stateDir, repositorykey+'.ids')

def ensureDirectory(directoryPath):
    isdir(directoryPath) or makedirs(directoryPath)

class HarvesterLog(object):
    def __init__(self, stateDir, logDir, name):
        self._name=name
        ensureDirectory(stateDir)
        self._ids = Ids(stateDir, name)
        self._state = State(stateDir, name)
        self._eventlogger = EventLogger(logDir + '/' + name +'.events')
        self.from_    = self._state.startdate
        self.fromhour = self._state.starthour
        self.token = self._state.token
        self.total = self._state.total
        self._lastline = ''

    def isCurrentDay(self, yyyy_mm_dd):
        return yyyy_mm_dd == self._state.getTime()[:10]    
        
    def isCurrentHour(self, yyyy_mm_dd, hour):
        return self.isCurrentDay(yyyy_mm_dd) and hour == self._state.getTime().split()[1][0:2]

    def startRepository(self):
        self._state._write('Started: %s, Harvested/Uploaded/Deleted/Total: ' % self._state.getTime())

    def totalids(self):
        return self._ids.total()

    def eventLogger(self):
        return self._eventlogger
            
    def markDeleted(self):
        self._ids.clear()
        self._state.markDeleted()
        self._eventlogger.succes('Harvested/Uploaded/Deleted/Total: 0/0/0/0, Done: Deleted all id\'s.',id=self._name)
    
    # FIXME TP: oairequesturl is logged
    def endRepository(self, url, token):
        self._state._write(self._lastline)
        self._state._write(', Done: %s, Request: %s, ResumptionToken: %s' % (self._state.getTime(), url, token))
        self._eventlogger.succes('Harvested/Uploaded/Deleted/Total: %s, ResumptionToken: %s'%(self._lastline,token),id=self._name)

    def endWithException(self):
        error = str(sys.exc_type) + ': ' + str(sys.exc_value)
        xtype,xval,xtb = sys.exc_info()
        error2 = '|'.join(map(str.strip,traceback.format_exception(xtype,xval,xtb)))
        self._eventlogger.error(error2, id=self._name)
        self._state._write(self._lastline)
        self._state._write( ', Error: ' + error)

    def close(self):
        self._eventlogger.close()
        self._ids.close()
        self._state.close()

    def logID(self, uploadid):
        self._ids.add(uploadid)
        
    def logDeletedID(self, uploadid):
        self._ids.remove(uploadid)
        
    def updateStatsfile(self, harvested, uploaded, deleted, totalWillBeIgnored=None):
        self._lastline = '%d/%d/%d/%d' % (harvested, uploaded, deleted, self.totalids())

    def hasWork(self):
        return not self.isCurrentHour(self.from_, self.fromhour) or self.token
